from __future__ import annotations

import dataclasses
import uuid

from onesecondtrader import models
from . import bases


@dataclasses.dataclass(kw_only=True, frozen=True)
class OrderSubmissionAccepted(bases.BrokerResponseEvent):
    broker_order_id: str | None = None


@dataclasses.dataclass(kw_only=True, frozen=True)
class OrderSubmissionRejected(bases.BrokerResponseEvent):
    reason: str | None = None


@dataclasses.dataclass(kw_only=True, frozen=True)
class OrderModificationAccepted(bases.BrokerResponseEvent):
    broker_order_id: str | None = None


@dataclasses.dataclass(kw_only=True, frozen=True)
class OrderModificationRejected(bases.BrokerResponseEvent):
    reason: str | None = None


@dataclasses.dataclass(kw_only=True, frozen=True)
class OrderCancellationAccepted(bases.BrokerResponseEvent):
    pass


@dataclasses.dataclass(kw_only=True, frozen=True)
class OrderCancellationRejected(bases.BrokerResponseEvent):
    reason: str | None = None


@dataclasses.dataclass(kw_only=True, frozen=True)
class OrderFilled(bases.BrokerResponseEvent):
    fill_id: uuid.UUID = dataclasses.field(default_factory=uuid.uuid4)
    broker_fill_id: str | None = None
    symbol: str
    side: models.orders.OrderSide
    quantity_filled: float
    fill_price: float
    commission: float
    exchange: str = "SIMULATED"


@dataclasses.dataclass(kw_only=True, frozen=True)
class OrderExpired(bases.BrokerResponseEvent):
    pass



import abc
import queue
import threading

from onesecondtrader import events
from .eventbus import EventBus


class Subscriber(abc.ABC):
    def __init__(self, event_bus: EventBus) -> None:
        self._event_bus = event_bus
        self._queue: queue.Queue[events.bases.EventBase | None] = queue.Queue()
        self._running: threading.Event = threading.Event()
        self._running.set()
        self._thread = threading.Thread(
            target=self._event_loop, name=self.__class__.__name__
        )
        self._thread.start()

    def receive(self, event: events.bases.EventBase) -> None:
        if self._running.is_set():
            self._queue.put(event)

    def wait_until_idle(self) -> None:
        if not self._running.is_set():
            return
        self._queue.join()

    def shutdown(self) -> None:
        if not self._running.is_set():
            return
        self._event_bus.unsubscribe(self)
        self._running.clear()
        self._queue.put(None)
        if threading.current_thread() is not self._thread:
            self._thread.join()

    def _subscribe(self, *event_types: type[events.bases.EventBase]) -> None:
        for event_type in event_types:
            self._event_bus.subscribe(self, event_type)

    def _publish(self, event: events.bases.EventBase) -> None:
        self._event_bus.publish(event)

    def _event_loop(self) -> None:
        while True:
            event = self._queue.get()
            if event is None:
                self._queue.task_done()
                break
            try:
                self._on_event(event)
            except Exception as exc:
                self._on_exception(exc)
            finally:
                self._queue.task_done()
        self._cleanup()

    def _on_exception(self, exc: Exception) -> None:
        # Override in subclass to log or handle exceptions
        pass

    def _cleanup(self) -> None:
        pass

    @abc.abstractmethod
    def _on_event(self, event: events.bases.EventBase) -> None:
        # Must not block indefinitely; wait_until_idle() has no timeout
        ...



from __future__ import annotations

import collections
import threading
import typing

from onesecondtrader import events

if typing.TYPE_CHECKING:
    from .subscriber import Subscriber


class EventBus:
    def __init__(self) -> None:
        self._per_event_subscriptions: collections.defaultdict[
            type[events.bases.EventBase], set[Subscriber]
        ] = collections.defaultdict(set)
        self._subscribers: set[Subscriber] = set()
        self._lock: threading.Lock = threading.Lock()

    def subscribe(
        self,
        subscriber: Subscriber,
        event_type: type[events.bases.EventBase],
    ) -> None:
        with self._lock:
            self._subscribers.add(subscriber)
            self._per_event_subscriptions[event_type].add(subscriber)

    def unsubscribe(self, subscriber: Subscriber) -> None:
        with self._lock:
            for set_of_event_subscribers in self._per_event_subscriptions.values():
                set_of_event_subscribers.discard(subscriber)
            self._subscribers.discard(subscriber)

    def publish(self, event: events.bases.EventBase) -> None:
        with self._lock:
            subscribers = self._per_event_subscriptions[type(event)].copy()
        for subscriber in subscribers:
            subscriber.receive(event)

    def wait_until_system_idle(self) -> None:
        with self._lock:
            subscribers = self._subscribers.copy()
        for subscriber in subscribers:
            subscriber.wait_until_idle()



from __future__ import annotations

import dataclasses

from onesecondtrader import models
from . import bases


@dataclasses.dataclass(kw_only=True, frozen=True)
class BarReceived(bases.MarketEvent):
    symbol: str
    bar_period: models.data.BarPeriod
    open: float
    high: float
    low: float
    close: float
    volume: int | None = None


@dataclasses.dataclass(kw_only=True, frozen=True)
class BarProcessed(BarReceived):
    indicators: dict[str, float] = dataclasses.field(default_factory=dict)
